package net.sf.sido.spring;

import java.text.MessageFormat;
import java.text.ParseException;
import java.util.Comparator;

import org.springframework.beans.factory.annotation.Required;
import org.springframework.core.io.Resource;

public class ResourcePatternComparator<T extends Comparable<T>> implements Comparator<Resource> {

	private String pattern;
	private int index = 0;
	private boolean ascending = true;

	@Override
	public int compare(Resource o1, Resource o2) {
		// Gets the resource names
		String n1 = o1.getFilename();
		String n2 = o2.getFilename();
		// Parses the names
		MessageFormat format = new MessageFormat(pattern);
		try {
			Object[] t1 = format.parse(n1);
			Object[] t2 = format.parse(n2);
			// Gets the sorting information
			@SuppressWarnings("unchecked")
			T c1 = (T) t1[index];
			@SuppressWarnings("unchecked")
			T c2 = (T) t2[index];
			// Comparison
			int result = c1.compareTo(c2);
			// Direction
			if (!ascending) {
				result = -result;
			}
			// OK
			return result;
		} catch (ParseException ex) {
			throw new IllegalArgumentException(String.format("Cannot parse resource names [%s] and [%s] with pattern [%s]",n1, n2, pattern ), ex);
		}
	}

	public String getPattern() {
		return pattern;
	}

	@Required
	public void setPattern(String pattern) {
		this.pattern = pattern;
	}

	public int getIndex() {
		return index;
	}

	public void setIndex(int index) {
		this.index = index;
	}

	public boolean isAscending() {
		return ascending;
	}

	public void setAscending(boolean ascending) {
		this.ascending = ascending;
	}

}
